#!/usr/bin/env python
# -*- coding: utf-8 -*-

import params
import pytest

"""
Test command "open-project --set-default-variant"
"""


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_project_with_two_boards_save(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--set-default-variant=AV',
                                   '--save',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Set default assembly variant to 'AV'...\n" \
        "Save project...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0

    # Open again to check that the assembly variant still exists.
    code, stdout, stderr = cli.run('open-project',
                                   '--set-default-variant=AV',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Set default assembly variant to 'AV'...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0


@pytest.mark.parametrize("project", [params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM])
def test_invalid_variant(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--set-default-variant=Foo',
                                   '--save',
                                   project.path)
    assert stderr == \
        "ERROR: No assembly variant with the name 'Foo' found.\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Set default assembly variant to 'Foo'...\n" \
        "Save project...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import subprocess

DATA_ROOT = os.path.dirname(os.path.realpath(__file__))
CLI_EXECUTABLE = sys.argv[1]


def find(suffix):
    result = []
    for root, dirnames, filenames in os.walk(DATA_ROOT):
        if root.endswith(suffix):
            result.append(root)
        for filename in filenames:
            if filename.endswith(suffix):
                result.append(os.path.join(root, filename))
    return set(result)


def update_libraries():
    for library in find('.lplib'):
        subprocess.check_call([
            CLI_EXECUTABLE,
            'open-library',
            '--all',
            '--save',
            library,
        ])


def update_projects():
    for project in find('.lpp'):
        subprocess.check_call([
            CLI_EXECUTABLE,
            'open-project',
            '--save',
            project,
        ])


if __name__ == "__main__":
    os.environ['LIBREPCB_DISABLE_UNSTABLE_WARNING'] = '1'
    os.environ['LIBREPCB_UPGRADE_UNSTABLE'] = '1'
    update_libraries()
    update_projects()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import shutil

"""
Test command "open-library --check"
"""


def test_no_messages(cli):
    library = params.EMPTY_LIBRARY
    cli.add_library(library.dir)
    code, stdout, stderr = cli.run('open-library', '--all', '--check',
                                   library.dir)
    assert stderr == ''
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "SUCCESS\n".format(library=library)
    assert code == 0


def test_messages(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    for subdir in ['sym', 'pkg', 'cmp']:
        shutil.rmtree(cli.abspath(os.path.join(library.dir, subdir)))
    code, stdout, stderr = cli.run('open-library', '--all', '--check',
                                   library.dir)
    assert stderr == \
        "  - R-0805 (078650d3-483c-4b9e-a848-b14f1aad2edc):\n" \
        "    - [HINT] No part numbers added\n" \
        "  - R-0603 (483a71eb-318e-448e-82ff-f02efc4821aa):\n" \
        "    - [HINT] No part numbers added\n" \
        "  - PSMN022-30PL (5738d8f9-4101-4409-bd46-d9c173b40d60):\n" \
        "    - [ERROR] No categories set\n" \
        "    - [HINT] No part numbers added\n" \
        "  - R-1206 (a6a6744d-7d3b-450a-b782-feca43939ca5):\n" \
        "    - [HINT] No part numbers added\n" \
        "  - C-0805 (c139e505-592b-46ba-bdf2-acb7383ea0cd):\n" \
        "    - [HINT] No part numbers added\n" \
        "  - PSMN5R8 (f7fb22e8-0bbc-4f0f-aa89-596823b5bc3e):\n" \
        "    - [ERROR] No categories set\n" \
        "    - [HINT] No part numbers added\n" \
        "  - Crystal ABM3 (f83a5ae8-7f42-42be-9dd6-e762f4da2ec2):\n" \
        "    - [HINT] No part numbers added\n"
    assert stdout == \
        "Open library 'Populated Library.lplib'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process 0 symbols...\n" \
        "Process 0 packages...\n" \
        "Process 0 components...\n" \
        "Process {library.dev} devices...\n" \
        "Finished with errors!\n".format(library=library)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --export-schematics"
"""


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_if_unknown_file_extension_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-schematics=foo.bar',
                                   project.path)
    assert stderr == \
        "  ERROR: Failed to export image \"{path}\". Check file permissions " \
        "and make sure to use a supported image file extension.\n".format(
            path=cli.abspath('foo.bar'),
        )
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export schematics to 'foo.bar'...\n" \
        "Finished with errors!\n".format(project=project)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_exporting_pdf_with_relative_path(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    path = cli.abspath('sch.pdf')
    assert not os.path.exists(path)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-schematics=sch.pdf',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export schematics to 'sch.pdf'...\n" \
        "  => 'sch.pdf'\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0
    assert os.path.exists(path)


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_exporting_pdf_with_absolute_path(cli, project):
    """
    Note: Test with passing the argument as "--arg <value>".
    """
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    path = cli.abspath('schematic with spaces.pdf')
    assert not os.path.exists(path)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-schematics', path,
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export schematics to '{path}'...\n" \
        "  => '{path}'\n" \
        "SUCCESS\n".format(project=project, path=path)
    assert code == 0
    assert os.path.exists(path)


@pytest.mark.parametrize("file_extension", [
    'svg',
    'png',
])
def test_exporting_images(cli, file_extension):
    project = params.PROJECT_WITH_TWO_BOARDS_LPPZ
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    path = cli.abspath('schematic with spaces.' + file_extension)
    assert not os.path.exists(path)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-schematics={}'.format(path),
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export schematics to '{path}'...\n" \
        "  => '{path}'\n" \
        "SUCCESS\n".format(project=project, path=path)
    assert code == 0
    assert os.path.exists(path)


@pytest.mark.parametrize("project", [
    params.EMPTY_PROJECT_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_if_output_directories_are_created(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    dir = cli.abspath('nonexistent directory/nested')
    path = os.path.join(dir, 'schematic.pdf')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-schematics={}'.format(path),
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export schematics to '{path}'...\n" \
        "  => '{path}'\n" \
        "SUCCESS\n".format(project=project, path=path)
    assert code == 0
    assert os.path.exists(dir)
    assert os.path.exists(path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test the "Initialize Workspace Wizard" on a fresh installation, i.e.
without any workspace set in the user settings
"""

import os
import shutil


def test_env_default_path(librepcb, helpers):
    """
    Set the LIBREPCB_DEFAULT_WORKSPACE_PATH environment variable and check
    if the default workspace path is set to this value
    """
    test_workspace_path = librepcb.workspace_path
    librepcb.env['LIBREPCB_DEFAULT_WORKSPACE_PATH'] = test_workspace_path
    librepcb.workspace_path = None  # Do not set workspace in LibrePCB.ini
    with librepcb.open() as app:
        # Skip "Welcome to LibrePCB" page.
        app.widget('initWorkspaceWizardNextButton').click()

        # Check workspace path.
        path_edit = app.widget('initWorkspaceWizardChooseWorkspacePathEdit')
        assert path_edit.properties()['text'] == test_workspace_path
        status_label = app.widget('initWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']


def test_create_new_workspace(librepcb, helpers):
    """
    Choose a non-existent workspace path to create a new workspace
    """
    test_workspace_path = librepcb.workspace_path + '-test'
    librepcb.workspace_path = None  # Do not set workspace in LibrePCB.ini
    with librepcb.open() as app:
        # Skip "Welcome to LibrePCB" page.
        wizard = app.widget('initWorkspaceWizard')
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace path.
        path_edit = app.widget('initWorkspaceWizardChooseWorkspacePathEdit')
        default_path = os.path.join(os.path.expanduser('~'), 'LibrePCB-Workspace')
        assert path_edit.properties()['text'] == default_path
        path_edit.set_property('text', test_workspace_path)
        status_label = app.widget('initWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'workspace will be created' in status_label.properties()['text']
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace settings.
        user_edit = app.widget('initWorkspaceWizardChooseSettingsUserNameEdit')
        user_edit.set_property('text', 'foobar 42')
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the workspace has been created.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(os.path.join(test_workspace_path,
                                           '.librepcb-workspace'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened
    # and the settings are applied.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True
        app.action('controlPanelActionOpenWorkspaceSettings').trigger(blocking=False)
        user_edit = app.widget('controlPanelWorkspaceSettingsDialogGeneralUserNameEdit')
        assert user_edit.properties()['text'] == 'foobar 42'


def test_open_compatible_workspace(librepcb):
    """
    Choose a compatible workspace to open
    """
    test_workspace_path = librepcb.workspace_path
    librepcb.workspace_path = None  # Do not set workspace in LibrePCB.ini
    with librepcb.open() as app:
        # Skip "Welcome to LibrePCB" page.
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace path.
        path_edit = app.widget('initWorkspaceWizardChooseWorkspacePathEdit')
        default_path = os.path.join(os.path.expanduser('~'), 'LibrePCB-Workspace')
        assert path_edit.properties()['text'] == default_path
        path_edit.set_property('text', test_workspace_path)
        status_label = app.widget('initWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True


def test_open_workspace_upgrade_v01(librepcb, helpers):
    """
    Choose a workspace to open, which needs to be upgraded first, by
    copying 'v0.1' to 'data'
    """
    test_workspace_path = librepcb.workspace_path
    librepcb.workspace_path = None  # Do not set workspace in LibrePCB.ini
    v01_path = os.path.join(test_workspace_path, 'v0.1')
    data_path = os.path.join(test_workspace_path, 'data')
    shutil.rmtree(data_path)
    with librepcb.open() as app:
        # Skip "Welcome to LibrePCB" page.
        wizard = app.widget('initWorkspaceWizard')
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace path.
        path_edit = app.widget('initWorkspaceWizardChooseWorkspacePathEdit')
        default_path = os.path.join(os.path.expanduser('~'), 'LibrePCB-Workspace')
        assert path_edit.properties()['text'] == default_path
        path_edit.set_property('text', test_workspace_path)
        status_label = app.widget('initWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']
        app.widget('initWorkspaceWizardNextButton').click()

        # Perform upgrade.
        src_label = app.widget('initWorkspaceWizardUpgradeSourceLabel')
        assert v01_path in src_label.properties()['text']
        dst_label = app.widget('initWorkspaceWizardUpgradeDestinationLabel')
        assert data_path in dst_label.properties()['text']
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the data directory has been created.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(os.path.join(data_path, '.librepcb-data'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True


def test_open_workspace_upgrade_data(librepcb, helpers):
    """
    Choose a workspace to open, which needs to be upgraded first, by
    copying 'data' to 'v0.1'
    """
    test_workspace_path = librepcb.workspace_path
    librepcb.workspace_path = None  # Do not set workspace in LibrePCB.ini
    v01_path = os.path.join(test_workspace_path, 'v0.1')
    data_path = os.path.join(test_workspace_path, 'data')
    shutil.rmtree(v01_path)
    os.remove(os.path.join(data_path, '.librepcb-data'))  # Downgrade to v0.1
    with librepcb.open() as app:
        # Skip "Welcome to LibrePCB" page.
        wizard = app.widget('initWorkspaceWizard')
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace path.
        path_edit = app.widget('initWorkspaceWizardChooseWorkspacePathEdit')
        default_path = os.path.join(os.path.expanduser('~'), 'LibrePCB-Workspace')
        assert path_edit.properties()['text'] == default_path
        path_edit.set_property('text', test_workspace_path)
        status_label = app.widget('initWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']
        app.widget('initWorkspaceWizardNextButton').click()

        # Perform upgrade.
        src_label = app.widget('initWorkspaceWizardUpgradeSourceLabel')
        assert data_path in src_label.properties()['text']
        dst_label = app.widget('initWorkspaceWizardUpgradeDestinationLabel')
        assert v01_path in dst_label.properties()['text']
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the v0.1 directory has been created and the data
        # directory has been upgraded.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(v01_path)
        assert os.path.exists(os.path.join(data_path, '.librepcb-data'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True


def test_open_workspace_downgrade(librepcb, helpers):
    """
    Choose a workspace to open, which needs to be initialized with an
    older file format
    """
    test_workspace_path = librepcb.workspace_path
    librepcb.workspace_path = None  # Do not set workspace in LibrePCB.ini
    v01_path = os.path.join(test_workspace_path, 'v0.1')
    v02_path = os.path.join(test_workspace_path, 'v1')
    data_path = os.path.join(test_workspace_path, 'data')
    shutil.rmtree(v01_path)
    with open(os.path.join(data_path, '.librepcb-data'), 'wb') as f:
        f.write(b'999\n')
    with librepcb.open() as app:
        # Skip "Welcome to LibrePCB" page.
        wizard = app.widget('initWorkspaceWizard')
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace path.
        path_edit = app.widget('initWorkspaceWizardChooseWorkspacePathEdit')
        default_path = os.path.join(os.path.expanduser('~'), 'LibrePCB-Workspace')
        assert path_edit.properties()['text'] == default_path
        path_edit.set_property('text', test_workspace_path)
        status_label = app.widget('initWorkspaceWizardChooseWorkspaceStatusLabel')
        assert 'contains a valid workspace' in status_label.properties()['text']
        app.widget('initWorkspaceWizardNextButton').click()

        # Choose workspace settings.
        user_edit = app.widget('initWorkspaceWizardChooseSettingsUserNameEdit')
        user_edit.set_property('text', 'foobar 42')
        app.widget('initWorkspaceWizardFinishButton').click()

        # Verify that the v1 directory has been created.
        helpers.wait_until_widget_hidden(wizard)
        assert os.path.exists(os.path.join(v02_path, '.librepcb-data'))

        # Verify that the control panel is now opened.
        app.widget('controlPanel').properties()['visible'] is True

    # Open LibrePCB again to see if the workspace is automatically opened.
    with librepcb.open() as app:
        app.widget('controlPanel').properties()['visible'] is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --remove-other-boards"
"""


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_project_with_two_boards_remove_both_volatile(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    # Without passing '--save', the removal is not persistent.
    for _ in range(2):
        code, stdout, stderr = cli.run('open-project',
                                       '--remove-other-boards',
                                       project.path)
        assert stderr == ''
        assert stdout == \
            "Open project '{project.path}'...\n" \
            "Remove other boards...\n" \
            "  - 'default'\n" \
            "  - 'copy'\n" \
            "SUCCESS\n".format(project=project)
        assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_project_with_two_boards_remove_one_and_save(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    removed_dir = cli.abspath(project.dir + '/boards/default')
    if not project.is_lppz:
        assert os.path.exists(removed_dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--board=copy',
                                   '--remove-other-boards',
                                   '--save',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Remove other boards...\n" \
        "  - 'default'\n" \
        "Save project...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0
    if not project.is_lppz:
        assert os.path.exists(removed_dir) is False

    # Open again to check that the removed board is not loaded anymore.
    code, stdout, stderr = cli.run('open-project',
                                   '--board=copy',
                                   '--remove-other-boards',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Remove other boards...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
import os

"""
Test image export dialog in symbol editor

Meant as an addition to the tests in test_schematic_image_export.py just to
test the symbol editor specific functionality works as well.
"""


@pytest.fixture
def dialog(library_editor, helpers):
    """
    Fixture opening the image export dialog in the symbol editor
    """
    list_widget = library_editor.widget('libraryEditorOverviewSymList')
    helpers.wait_for_model_items_count(list_widget, 1)
    item = list_widget.model().items().items[0]
    list_widget.dclick_item(item)

    library_editor.action('libraryEditorActionExportImage').\
        trigger(blocking=False)
    dialog = library_editor.widget('libraryEditorGraphicsExportDialog')

    # Do not open exported files since this would be annoying here.
    library_editor.widget(
        'libraryEditorGraphicsExportDialogOpenFileCheckbox').\
        set_property('checked', False)

    yield dialog


def test_export_png(dialog, library_editor, librepcb, helpers):
    """
    Test if the accept button asks for a filepath and then creates a PNG file
    """
    library_editor.widget(
        'libraryEditorGraphicsExportDialogButtonAccept').click()
    path = librepcb.abspath('test export.png')
    library_editor.widget(
        'libraryEditorGraphicsExportSaveFileDialogFileNameEdit').\
        set_property('text', path)
    library_editor.widget(
        'libraryEditorGraphicsExportSaveFileDialogOkButton').click()
    helpers.wait_until_widget_hidden(dialog)  # raises on timeout
    assert os.path.exists(path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test command "open-step" (basic parser tests)
"""

HELP_TEXT = """\
Usage: {executable} [options] open-step [command_options] file
LibrePCB Command Line Interface

Options:
  -h, --help        Print this message.
  -V, --version     Displays version information.
  -v, --verbose     Verbose output.
  --minify          Minify the STEP model before validating it. Use in
                    conjunction with '--save-to' to save the output of the
                    operation.
  --tesselate       Tesselate the loaded STEP model to check if LibrePCB is
                    able to render it. Reports failure (exit code = 1) if no
                    content is detected.
  --save-to <file>  Write the (modified) STEP file to this output location (may
                    be equal to the opened file path). Only makes sense in
                    conjunction with '--minify'.

Arguments:
  open-step         Open a STEP model to execute STEP-related tasks outside of
                    a library.
  file              Path to the STEP file (*.step).
"""

ERROR_TEXT = """\
{error}
Usage: {executable} [options] open-step [command_options] file
Help: {executable} open-step --help
"""


def test_help(cli):
    code, stdout, stderr = cli.run('open-step', '--help')
    assert stderr == ''
    assert stdout == HELP_TEXT.format(executable=cli.executable)
    assert code == 0


def test_no_arguments(cli):
    code, stdout, stderr = cli.run('open-step')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Missing arguments: file",
    )
    assert stdout == ''
    assert code == 1


def test_invalid_argument(cli):
    code, stdout, stderr = cli.run('open-step', '--invalid-argument')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Unknown option 'invalid-argument'.",
    )
    assert stdout == ''
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params

"""
Test command "open-library --strict"
"""


def test_valid_lp(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    code, stdout, stderr = cli.run('open-library', '--all', '--strict',
                                   library.dir)
    assert stderr == ''
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "SUCCESS\n".format(library=library)
    assert code == 0


def test_invalid_lp(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    # append some spaces to the library file and a symbol file
    paths = [
        library.dir + '/library.lp',
        library.dir + '/sym/9b75d0ce-ac4e-4a52-a88a-8777f66d3241/symbol.lp',
    ]
    for path in paths:
        with open(cli.abspath(path), 'ab') as f:
            f.write(b'  ')
    # open library
    code, stdout, stderr = cli.run('open-library', '--all', '--strict',
                                   library.dir)
    assert stderr == \
        "  - Populated Library (a7cb5051-9f37-4500-be38-33eade6e621e):\n" \
        "    - Non-canonical file: '{paths[0]}'\n" \
        "  - Diode (9b75d0ce-ac4e-4a52-a88a-8777f66d3241):\n" \
        "    - Non-canonical file: '{paths[1]}'\n" \
        .format(paths=paths).replace('/', os.sep)
    assert stdout == \
        "Open library 'Populated Library.lplib'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "Finished with errors!\n".format(library=library)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test command "open-library" (basic parser tests)
"""

HELP_TEXT = """\
Usage: {executable} [options] open-library [command_options] library
LibrePCB Command Line Interface

Options:
  -h, --help     Print this message.
  -V, --version  Displays version information.
  -v, --verbose  Verbose output.
  --all          Perform the selected action(s) on all elements contained in
                 the opened library.
  --check        Run the library element check, print all non-approved messages
                 and report failure (exit code = 1) if there are non-approved
                 messages.
  --minify-step  Minify the STEP models of all packages. Only works in
                 conjunction with '--all'. Pass '--save' to write the minified
                 files to disk.
  --save         Save library (and contained elements if '--all' is given)
                 before closing them (useful to upgrade file format).
  --strict       Fail if the opened files are not strictly canonical, i.e.
                 there would be changes when saving the library elements.

Arguments:
  open-library   Open a library to execute library-related tasks.
  library        Path to library directory (*.lplib).
"""

ERROR_TEXT = """\
{error}
Usage: {executable} [options] open-library [command_options] library
Help: {executable} open-library --help
"""


def test_help(cli):
    code, stdout, stderr = cli.run('open-library', '--help')
    assert stderr == ''
    assert stdout == HELP_TEXT.format(executable=cli.executable)
    assert code == 0


def test_no_arguments(cli):
    code, stdout, stderr = cli.run('open-library')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Missing arguments: library",
    )
    assert stdout == ''
    assert code == 1


def test_invalid_argument(cli):
    code, stdout, stderr = cli.run('open-library', '--invalid-argument')
    assert stderr == ERROR_TEXT.format(
        executable=cli.executable,
        error="Unknown option 'invalid-argument'.",
    )
    assert stdout == ''
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test importing an EAGLE library with the library editor
"""


def _get_elements_count(le):
    symbols = len(le.widget('libraryEditorOverviewSymList').model().items().items)
    packages = len(le.widget('libraryEditorOverviewPkgList').model().items().items)
    components = len(le.widget('libraryEditorOverviewCmpList').model().items().items)
    devices = len(le.widget('libraryEditorOverviewDevList').model().items().items)
    return symbols, packages, components, devices


def test(librepcb, library_editor, helpers):
    """
    Import EAGLE library
    """
    le = library_editor
    lbr = librepcb.get_data_path('unittests/eagleimport/resistor.lbr')

    # Get current library elements count
    symbols, packages, components, devices = _get_elements_count(le)

    # Open "EAGLE Import" wizard
    le.action('libraryEditorActionImportEagleLibrary').trigger(blocking=False)
    next_button = le.widget('libraryEditorEagleLibraryImportWizardNextButton')

    # Choose library - first an invalid path, then a valid path
    next_button.click()
    path_edit = le.widget('libraryEditorEagleLibraryImportWizardChooseLibraryFilePathEdit')
    assert next_button.properties()['enabled'] is False
    path_edit.set_property('text', lbr + "_")
    assert next_button.properties()['enabled'] is False
    path_edit.set_property('text', lbr)
    assert next_button.properties()['enabled'] is True

    # Choose elements to import
    next_button.click()
    tree_widget = le.widget('libraryEditorEagleLibraryImportWizardSelectElementsTreeWidget')
    helpers.wait_for_model_items_count(tree_widget, 4, 4)
    component = tree_widget.model().items().items[0]
    tree_widget.select_item(component)
    assert next_button.properties()['enabled'] is False
    tree_widget.shortcut('Space')  # Select item
    assert next_button.properties()['enabled'] is True
    tree_widget.shortcut('Space')  # Deselect item
    assert next_button.properties()['enabled'] is False
    tree_widget.shortcut('Space')  # Select item
    assert next_button.properties()['enabled'] is True

    # Set import options
    next_button.click()
    add_prefix_checkbox = le.widget('libraryEditorEagleLibraryImportWizardSetOptionsPrefixCheckBox')
    add_prefix_checkbox.click()
    choose_cmpcat_button = le.widget('libraryEditorEagleLibraryImportWizardSetOptionsComponentCategoryChooseButton')
    choose_cmpcat_button.click()
    category_tree = le.widget('libraryEditorEagleLibraryImportWizardSetOptionsChooseCategoryTree')
    helpers.wait_for_model_items_count(category_tree, 1)
    category = category_tree.model().items().items[0]
    category_tree.select_item(category)
    category_tree.shortcut('Return')

    # Import
    import_button = le.widget('libraryEditorEagleLibraryImportWizardImportButton')
    import_button.click()
    finish_button = le.widget('libraryEditorEagleLibraryImportWizardFinishButton')
    finish_button.wait_for_properties(dict(enabled=True))
    finish_button.click()

    # Check if elements were added to library
    helpers.wait_for_library_scan_complete(le)
    assert _get_elements_count(le) == \
        (symbols + 1, packages + 1, components + 1, devices + 1)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --export-board-bom"
"""


@pytest.mark.parametrize("project", [params.EMPTY_PROJECT_LPP_PARAM])
def test_if_project_without_boards_succeeds(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)

    # remove all boards first
    with open(cli.abspath(project.dir + '/boards/boards.lp'), 'w') as f:
        f.write('(librepcb_boards)')

    relpath = project.output_dir + '/bom/bom.csv'
    abspath = cli.abspath(relpath)
    assert not os.path.exists(abspath)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom=' + relpath,
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to 'Empty Project/output/v1/bom/bom.csv'...\n" \
        "SUCCESS\n".format(project=project)
    assert code == 0
    assert not os.path.exists(abspath)  # nothing exported


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_export_project_with_two_boards_implicit(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp = project.output_dir + '/bom/{{BOARD}}.csv'
    dir = cli.abspath(project.output_dir + '/bom')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom=' + fp,
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{project.output_dir}/bom/{{{{BOARD}}}}.csv'...\n" \
        "  - 'default' => '{project.output_dir_native}//bom//default.csv'\n" \
        "  - 'copy' => '{project.output_dir_native}//bom//copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 2


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
    params.PROJECT_WITH_TWO_BOARDS_LPPZ_PARAM,
])
def test_export_project_with_two_boards_explicit_one_attributes(cli, project):
    """
    Note: Test with passing the argument as "--arg <value>".
    """
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp = project.output_dir + '/bom/{{BOARD}}.csv'
    dir = cli.abspath(project.output_dir + '/bom')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom', fp,
                                   '--board=copy',
                                   '--bom-attributes', 'SUPPLIER, SKU',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{project.output_dir}/bom/{{{{BOARD}}}}.csv'...\n" \
        "  - 'copy' => '{project.output_dir_native}//bom//copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert os.listdir(dir) == ['copy.csv']
    fp = cli.abspath(fp.replace('{{BOARD}}', 'copy'))
    assert b'SUPPLIER,SKU' in open(fp, 'rb').read()


@pytest.mark.parametrize("project", [params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM])
def test_export_project_with_two_boards_with_variant(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp = project.output_dir + '/bom/{{BOARD}}_{{VARIANT}}.csv'
    dir = cli.abspath(project.output_dir + '/bom')
    assert not os.path.exists(dir)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom', fp,
                                   '--variant-index=0',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{project.output_dir}/bom/{{{{BOARD}}}}_{{{{VARIANT}}}}.csv'...\n" \
        "  - 'default' => '{project.output_dir_native}//bom//default_AV.csv'\n" \
        "  - 'copy' => '{project.output_dir_native}//bom//copy_AV.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0
    assert os.path.exists(dir)
    assert len(os.listdir(dir)) == 2


@pytest.mark.parametrize("project", [params.PROJECT_WITH_TWO_BOARDS_LPP])
def test_export_project_with_two_conflicting_boards_fails(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    fp = project.output_dir + '/bom.csv'
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom=' + fp,
                                   project.path)
    assert stderr == \
        "ERROR: The file '{project.output_dir_native}//bom.csv' was " \
        "written multiple times!\n" \
        "NOTE: To avoid writing files multiple times, make sure to pass " \
        "unique filepaths to all export functions. For board output files, " \
        "you could either add the placeholder '{{{{BOARD}}}}' to the path " \
        "or specify the boards to export with the '--board' argument.\n" \
        .format(project=project).replace('//', os.sep)
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{project.output_dir}/bom.csv'...\n" \
        "  - 'default' => '{project.output_dir_native}//bom.csv'\n" \
        "  - 'copy' => '{project.output_dir_native}//bom.csv'\n" \
        "Finished with errors!\n".format(project=project).replace('//', os.sep)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params

"""
Test command "open-library --save"
"""


def test_save(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    # append some spaces to the library file and a symbol file
    paths = [
        cli.abspath(library.dir + '/library.lp'),
        cli.abspath(library.dir + '/sym/9b75d0ce-ac4e-4a52-a88a-8777f66d3241/symbol.lp'),
    ]
    for path in paths:
        with open(path, 'ab') as f:
            f.write(b'  ')
    original_filesizes = [os.path.getsize(path) for path in paths]
    # save library (must remove the appended zeros)
    code, stdout, stderr = cli.run('open-library', '--all', '--save',
                                   library.dir)
    assert stderr == ''
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "SUCCESS\n".format(library=library)
    assert code == 0
    filesizes = [os.path.getsize(path) for path in paths]
    assert all([s[0] == (s[1] - 2) for s in zip(filesizes, original_filesizes)])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test opening projects from within control panel
"""


def test_open_dialog(librepcb, helpers):
    """
    Open project by open dialog in control panel
    """
    librepcb.add_project('Empty Project')
    with librepcb.open() as app:
        path = librepcb.abspath('Empty Project/Empty Project.lpp')
        app.widget('controlPanelOpenProjectButton').click()
        app.widget('controlPanelOpenProjectDialogFileNameEdit').set_property('text', path)
        app.widget('controlPanelOpenProjectDialogOkButton').click()

        # Check if both editors were opened
        assert app.widget('schematicEditor').properties()['visible'] is True
        assert app.widget('boardEditor').properties()['visible'] is True

        # Check if the schematic editor is the active window
        helpers.wait_for_active_window(app, app.widget('schematicEditor'))  # raises on timeout



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

library = 'libraries/Populated Library.lplib'


@pytest.fixture
def library_editor(librepcb, helpers):
    """
    Fixture opening the library editor with an empty library
    """
    librepcb.add_local_library_to_workspace(path=library)
    with librepcb.open() as app:
        # Wait until the library scan is finished
        helpers.wait_for_library_scan_complete(app)

        # Open library manager
        app.widget('controlPanelOpenLibraryManagerButton').click()
        assert app.widget('libraryManager').properties()['visible'] is True

        # Select the empty library in library list
        library_list = app.widget('libraryManagerInstalledLibrariesList')
        helpers.wait_for_model_items_count(library_list, 2)
        library_item = library_list.model().items().items[1]
        library_list.select_item(library_item)

        # Open library editor of empty library
        app.widget('libraryManagerLibraryInfoWidgetOpenEditorButton').click()
        assert app.widget('libraryEditor').properties()['visible'] is True

        # Start the actual test
        yield app



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-project --board --board-index"
"""


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_no_filter(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'default' => 'default.csv'\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_name(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board=copy',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_names(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board=copy',  # --arg=<value>
                                   '--board', 'default',  # --arg <value>
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "  - 'default' => 'default.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_invalid_name(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board=foo',
                                   project.path)
    assert stderr == "ERROR: No board with the name 'foo' found.\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "Finished with errors!\n".format(project=project).replace('//', os.sep)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_index(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board-index=1',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_indices(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board-index=1',  # --arg=<value>
                                   '--board-index', '0',  # --arg <value>
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "  - 'default' => 'default.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_invalid_index(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board-index=5',
                                   '--board-index=foo',
                                   project.path)
    assert stderr == \
        "ERROR: Board index '5' is invalid.\n" \
        "ERROR: Board index 'foo' is invalid.\n"
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "Finished with errors!\n".format(project=project).replace('//', os.sep)
    assert code == 1


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_name_and_index_same(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board=copy',
                                   '--board-index=1',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0


@pytest.mark.parametrize("project", [
    params.PROJECT_WITH_TWO_BOARDS_LPP_PARAM,
])
def test_filter_by_name_and_index_different(cli, project):
    cli.add_project(project.dir, as_lppz=project.is_lppz)
    code, stdout, stderr = cli.run('open-project',
                                   '--export-board-bom={{BOARD}}.csv',
                                   '--board=default',
                                   '--board-index=1',
                                   project.path)
    assert stderr == ''
    assert stdout == \
        "Open project '{project.path}'...\n" \
        "Export board-specific BOM to '{{{{BOARD}}}}.csv'...\n" \
        "  - 'default' => 'default.csv'\n" \
        "  - 'copy' => 'copy.csv'\n" \
        "SUCCESS\n".format(project=project).replace('//', os.sep)
    assert code == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest

library = 'libraries/Populated Library.lplib'


@pytest.fixture
def project_editor(librepcb):
    """
    Fixture opening the project editor with an empty project
    """
    librepcb.add_local_library_to_workspace(path=library)
    librepcb.add_project('Empty Project')
    librepcb.set_project('Empty Project/Empty Project.lpp')
    with librepcb.open() as app:
        # Check if both editors were opened
        assert app.widget('schematicEditor').properties()['visible'] is True
        assert app.widget('boardEditor').properties()['visible'] is True

        # Start the actual test
        yield app



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os

"""
Test manually downloading libraries with the library manager
"""


def test(librepcb, helpers):
    """
    Download library by URL
    """
    with librepcb.open() as app:

        # Open library manager
        app.widget('controlPanelOpenLibraryManagerButton').click()
        assert app.widget('libraryManager').properties()['visible'] is True

        # Make sure there is only one entry ("New Library") in the libraries list
        library_count_before = 1
        library_list = app.widget('libraryManagerInstalledLibrariesList')
        helpers.wait_for_model_items_count(library_list, library_count_before,
                                           library_count_before)

        # Make sure the target directory does not exist yet
        libdir = librepcb.get_workspace_libraries_path('local/base.lplib')
        assert not os.path.exists(libdir)

        # Download library
        app.widget('libraryManagerAddLibraryTabBar').set_current_tab(2)
        widget_properties = {
            ('UrlEdit', 'text'): 'http://localhost:50080/blobs/LibrePCB_Base.zip',
            ('DirectoryEdit', 'text'): 'base.lplib',
        }
        for (widget, property), value in widget_properties.items():
            app.widget('libraryManagerDownloadManually' + widget).set_property(property, value)
        app.widget('libraryManagerDownloadManuallyDownloadButton').click()

        # Check if one library is added
        library_count_after = library_count_before + 1
        helpers.wait_for_model_items_count(library_list, library_count_after,
                                           library_count_after)

        # Check if the library directory now exists
        assert os.path.exists(libdir)

        # Open the new library to check if everything is OK
        app.widget('libraryManagerLibraryInfoWidgetOpenEditorButton').click()
        assert app.widget('libraryEditor').properties()['visible'] is True



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import pytest
import os

"""
Test PDF export dialog in schematic editor
"""


@pytest.fixture
def dialog(project_editor):
    """
    Fixture opening the PDF export dialog in the schematic editor
    """
    project_editor.action('schematicEditorActionExportPdf').\
        trigger(blocking=False)
    dialog = project_editor.widget('schematicEditorGraphicsExportDialog')

    # Do not open exported files since this would be annoying here.
    project_editor.widget(
        'schematicEditorGraphicsExportDialogOpenFileCheckbox').\
        set_property('checked', False)

    yield dialog


def test_active_widget(dialog, project_editor, helpers):
    """
    Test if the dialog is the active widget after opening it
    """
    helpers.wait_for_active_dialog(project_editor, dialog)  # raises on timeout


def test_cancel_by_button(dialog, project_editor, helpers):
    """
    Test if the cancel button closes the dialog
    """
    project_editor.widget(
        'schematicEditorGraphicsExportDialogButtonCancel').click()
    helpers.wait_until_widget_hidden(dialog)  # raises on timeout


def test_cancel_by_escape(dialog, project_editor, helpers):
    """
    Test if the escape key closes the dialog
    """
    dialog.shortcut('Escape')
    helpers.wait_until_widget_hidden(dialog)  # raises on timeout


def test_restore_defaults(dialog, project_editor, librepcb, helpers):
    """
    Test if the restore defaults button resets the settings
    """
    cbx = project_editor.widget(
        'schematicEditorGraphicsExportDialogPageSizeComboBox')
    index = cbx.properties()['currentIndex']
    cbx.set_property('currentIndex', index + 1)
    assert cbx.properties()['currentIndex'] == index + 1
    project_editor.widget(
        'schematicEditorGraphicsExportDialogButtonRestoreDefaults').click()
    assert cbx.properties()['currentIndex'] == index


def test_export(dialog, project_editor, librepcb, helpers):
    """
    Test if the accept button asks for a filepath and then creates the PDF file
    """
    project_editor.widget(
        'schematicEditorGraphicsExportDialogButtonAccept').click()
    path = librepcb.abspath('test export.pdf')
    project_editor.widget(
        'schematicEditorGraphicsExportSaveFileDialogFileNameEdit').\
        set_property('text', path)
    project_editor.widget(
        'schematicEditorGraphicsExportSaveFileDialogOkButton').click()
    helpers.wait_until_widget_hidden(dialog)  # raises on timeout
    assert os.path.exists(path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import params
import pytest

"""
Test command "open-library --minify-step"
"""


def test_save(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    step = library.dir + \
        '/pkg/0eaf289c-166d-4bd9-a4ba-dbf6bbc76ef1' + \
        '/4e198b4d-b61a-47bd-a7ad-39b2f6dc77e9.step'
    path = cli.abspath(step)
    original_filesize = os.path.getsize(path)
    code, stdout, stderr = cli.run('open-library', '--all', '--minify-step',
                                   '--save', library.dir)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    filesize = os.path.getsize(path)
    assert stderr == ''
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "  - Minified '{step}' from {old} to {new} bytes\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "SUCCESS\n".format(library=library, step=step, old=original_filesize,
                           new=filesize).replace('/', os.sep)
    assert code == 0
    assert filesize < original_filesize


def test_strict(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    step = library.dir + \
        '/pkg/0eaf289c-166d-4bd9-a4ba-dbf6bbc76ef1' + \
        '/4e198b4d-b61a-47bd-a7ad-39b2f6dc77e9.step'
    path = cli.abspath(step)
    original_filesize = os.path.getsize(path)
    code, stdout, stderr = cli.run('open-library', '--all', '--minify-step',
                                   '--strict', library.dir)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == \
        "  - TO220AB (0eaf289c-166d-4bd9-a4ba-dbf6bbc76ef1):\n" \
        "    - Non-canonical file: '{step}'\n" \
        .format(step=step).replace('/', os.sep)
    assert "Minified '{}'".format(step).replace('/', os.sep) in stdout
    assert "Finished with errors!" in stdout
    assert code == 1
    filesize = os.path.getsize(path)
    assert filesize == original_filesize


def test_invalid_file(cli):
    library = params.POPULATED_LIBRARY
    cli.add_library(library.dir)
    step = library.dir + \
        '/pkg/0eaf289c-166d-4bd9-a4ba-dbf6bbc76ef1' + \
        '/4e198b4d-b61a-47bd-a7ad-39b2f6dc77e9.step'
    path = cli.abspath(step)
    with open(path, 'wb') as f:
        f.write(b'invalid')
    code, stdout, stderr = cli.run('open-library', '--all', '--minify-step',
                                   library.dir)
    if 'LibrePCB was compiled without OpenCascade' in stderr:
        pytest.skip("Feature not available.")
    assert stderr == \
        "  - TO220AB (0eaf289c-166d-4bd9-a4ba-dbf6bbc76ef1):\n" \
        "    - Failed to minify STEP model '{step}': STEP data section not found.\n" \
        .format(step=step).replace('/', os.sep)
    assert stdout == \
        "Open library '{library.dir}'...\n" \
        "Process {library.cmpcat} component categories...\n" \
        "Process {library.pkgcat} package categories...\n" \
        "Process {library.sym} symbols...\n" \
        "Process {library.pkg} packages...\n" \
        "Process {library.cmp} components...\n" \
        "Process {library.dev} devices...\n" \
        "Finished with errors!\n".format(library=library)
    assert code == 1



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Test opening the library manager
"""


def test_using_button(librepcb):
    """
    Open library manager with the button in the control panel
    """
    with librepcb.open() as app:
        app.widget('controlPanelOpenLibraryManagerButton').click()
        assert app.widget('libraryManager').properties()['visible'] is True


def test_using_menu(librepcb):
    """
    Open library manager with the menu item in the control panel
    """
    with librepcb.open() as app:
        app.action('controlPanelActionOpenLibraryManager').trigger()
        assert app.widget('libraryManager').properties()['visible'] is True


# TODO: How to emulate a click on the hyperlink in a QLabel?
# def test_using_no_libraries_warning(librepcb):
#     """
#     Open library manager with the link in the "no libraries installed" warning
#     """
#     with librepcb.open() as app:
#         label = app.widget('controlPanelWarnForNoLibrariesLabel')
#         assert app.widget('libraryManager').properties()['visible'] is True
